from onegov.core.orm import Base
from onegov.core.orm.mixins import ContentMixin
from onegov.core.orm.mixins import TimestampMixin
from onegov.core.orm.types import UUID
from onegov.search import ORMSearchable
from sqlalchemy import Column
from sqlalchemy import ForeignKey
from sqlalchemy import Integer
from sqlalchemy import Text
from sqlalchemy.orm import backref
from sqlalchemy.orm import relationship
from uuid import uuid4


class AgencyMembership(Base, ContentMixin, TimestampMixin, ORMSearchable):
    """ A membership to an agency. """

    __tablename__ = 'agency_memberships'

    #: the type of the item, this can be used to create custom polymorphic
    #: subclasses of this class. See
    #: `<http://docs.sqlalchemy.org/en/improve_toc/\
    #: orm/extensions/declarative/inheritance.html>`_.
    type = Column(Text, nullable=True)

    __mapper_args__ = {
        'polymorphic_on': type,
        'polymorphic_identity': None,
    }

    es_public = True
    es_properties = {
        'title': {'type': 'text'},
    }

    #: the unique id, part of the url
    id = Column(UUID, primary_key=True, default=uuid4)

    #: the id of the agency
    agency_id = Column(
        Integer,
        ForeignKey('agencies.id'),
        nullable=False
    )

    #: the related agency (which may have any number of memberships)
    agency = relationship(
        'Agency',
        backref=backref(
            'memberships',
            cascade='all, delete-orphan',
            lazy='dynamic',
            order_by='AgencyMembership.order'
        )
    )

    #: the id of the person
    person_id = Column(UUID, ForeignKey('people.id'), nullable=False)

    #: the related person (which may have any number of memberships)
    person = relationship(
        'Person',
        backref=backref(
            'memberships',
            cascade='all, delete-orphan',
            lazy='dynamic',
        )
    )

    #: the position of the membership within the agency
    order = Column(Integer, nullable=False)

    #: describes the membership
    title = Column(Text, nullable=False)

    #: when the membership started
    since = Column(Text, nullable=True)



from onegov.people.models.agency import Agency
from onegov.people.models.agency import AgencyOrganigram
from onegov.people.models.membership import AgencyMembership
from onegov.people.models.person import Person


__all__ = (
    'Agency',
    'AgencyMembership',
    'AgencyOrganigram',
    'Person',
)



from onegov.core.collection import GenericCollection
from onegov.people.models import AgencyMembership


class AgencyMembershipCollection(GenericCollection):
    """ Manages a list of agency memberships.

    Use it like this::

        from onegov.people import AgencyMembershipCollection
        memberships = AgencyMembershipCollection(session)

    """

    @property
    def model_class(self):
        return AgencyMembership

    def query(self):
        query = super(AgencyMembershipCollection, self).query()
        return query.order_by(self.model_class.order)



from onegov.people.models import Agency
from onegov.core.orm.abstract import AdjacencyListCollection


class AgencyCollection(AdjacencyListCollection):

    """ Manages a list of agencies.

    Use it like this::

        from onegov.people import AgencyCollection
        agencies = AgencyCollection(session)

    """

    __listclass__ = Agency

    @property
    def roots(self):
        """ Returns the root agencies. """

        return self.query().filter(
            self.__listclass__.parent_id.is_(None)
        ).all()
